--- wctdm.c.orig	2007-03-02 09:20:31.000000000 -0300
+++ wctdm.c	2007-03-02 15:20:23.000000000 -0300
@@ -323,6 +323,8 @@
 		        int lastpol;
 		        int polarity;
 		        int polaritydebounce;
+                int readcid;
+                unsigned int cidtimer;
 		} fxo;
 		struct {
 			int oldrxhook;
@@ -383,6 +385,7 @@
 static int fxorxgain = 0;
 static int fxstxgain = 0;
 static int fxsrxgain = 0;
+static int dtmf = 0;
 
 static int wctdm_init_proslic(struct wctdm *wc, int card, int fast , int manual, int sane);
 
@@ -464,6 +467,7 @@
 			if (!wc->mod[card].fxo.offhook)
 				zt_hooksig(&wc->chans[card], ZT_RXSIG_RING);
 			wc->mod[card].fxo.ring = 1;
+			wc->mod[card].fxo.readcid = 1;
 		}
 		if (wc->mod[card].fxo.ring && !wc->mod[card].fxo.pegcount) {
 			/* No more ring */
@@ -471,10 +475,35 @@
 				printk("NO RING on %d/%d!\n", wc->span.spanno, card + 1);
 			zt_hooksig(&wc->chans[card], ZT_RXSIG_OFFHOOK);
 			wc->mod[card].fxo.ring = 0;
+			wc->mod[card].fxo.cidtimer = wc->intcount;
+			wc->mod[card].fxo.readcid=0;
 		}
 	}
 }
 #endif
+static inline void wctdm_dtmfcheck_fakepolarity(struct wctdm *wc, int card, int x) 
+{
+	int sample;
+    /* only look for sound on the line if dtmf flag is on, it is an fxo card and line is onhook */ 
+	if (!dtmf || !(wc->cardflag & (1 << card)) || !(wc->modtype[card] == MOD_TYPE_FXO) || wc->mod[card].fxo.offhook ) 
+        return;
+
+    /* don't look for noise if we're already processing it, or there is a ringing tone */
+	if(!wc->mod[card].fxo.readcid && !wc->mod[card].fxo.wasringing  &&
+		wc->intcount > wc->mod[card].fxo.cidtimer + 400 ) {
+		sample = ZT_XLAW(wc->chans[card].readchunk[x], (&(wc->chans[card])));
+		if (sample > 16000 || sample < -16000) {
+			wc->mod[card].fxo.readcid = 1;
+			wc->mod[card].fxo.cidtimer = wc->intcount;
+			if (debug) printk("DTMF CLIP on %i\n",card+1);
+			zt_qevent_lock(&wc->chans[card], ZT_EVENT_POLARITY);
+		}
+	} else if(wc->mod[card].fxo.readcid && wc->intcount > wc->mod[card].fxo.cidtimer + 2000) {
+        /* reset flags if it's been a while */
+		wc->mod[card].fxo.cidtimer = wc->intcount;
+		wc->mod[card].fxo.readcid = 0;
+	}
+}
 static inline void wctdm_receiveprep(struct wctdm *wc, unsigned char ints)
 {
 	volatile unsigned int *readchunk;
@@ -505,6 +534,10 @@
 		if (wc->cardflag & (1 << 0))
 			wc->chans[0].readchunk[x] = (readchunk[x]) & 0xff;
 #endif
+		wctdm_dtmfcheck_fakepolarity(wc,0,x);
+		wctdm_dtmfcheck_fakepolarity(wc,1,x);
+		wctdm_dtmfcheck_fakepolarity(wc,2,x);
+		wctdm_dtmfcheck_fakepolarity(wc,3,x);
 	}
 #ifdef AUDIO_RINGCHECK
 	for (x=0;x<wc->cards;x++)
@@ -876,6 +909,8 @@
 			if (wc->mod[card].fxo.ringdebounce <= 0) {
 				if (wc->mod[card].fxo.wasringing) {
 					wc->mod[card].fxo.wasringing = 0;
+					wc->mod[card].fxo.readcid = 0;
+					wc->mod[card].fxo.cidtimer = wc->intcount;
 					zt_hooksig(&wc->chans[card], ZT_RXSIG_OFFHOOK);
 					if (debug)
 						printk("NO RING on %d/%d!\n", wc->span.spanno, card + 1);
@@ -2489,6 +2524,7 @@
 module_param(fxorxgain, int, 0600);
 module_param(fxstxgain, int, 0600);
 module_param(fxsrxgain, int, 0600);
+module_param(dtmf, int, 0600);
 #else
 MODULE_PARM(debug, "i");
 MODULE_PARM(loopcurrent, "i");
@@ -2509,6 +2545,7 @@
 MODULE_PARM(fxorxgain, "i");
 MODULE_PARM(fxstxgain, "i");
 MODULE_PARM(fxsrxgain, "i");
+MODULE_PARM(dtmf, "i");
 #endif
 MODULE_DESCRIPTION("Wildcard TDM400P Zaptel Driver");
 MODULE_AUTHOR("Mark Spencer <markster@digium.com>");
